--[[
    Baldrick_DailyQuests by LordFarlander
    Version 1.5.134
    $Revision: 131 $
]]--

--[[
Copyright (c) 2008, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

local Baldrick = _G["Baldrick"];

if( not Baldrick ) then
    error( L["%s requires %s"]:format( "Baldrick_DailyQuests", "Baldrick" ) );
    return;
end--if

local L = LibStub( "AceLocale-3.0" ):GetLocale( "Baldrick_DailyQuests" );
local LBZ = LibStub( "LibBabble-Zone-3.0" );
local SpecialEventsAura = LibStub( "LibLordFarlander-SpecialEvents-Aura-4.0" );

if( LBZ ) then
    LBZ = LBZ:GetUnstrictLookupTable();
end--if

local ZONES = {};
setmetatable( ZONES, {
    __index = function( self, key )
            return LBZ and LBZ[key] or L[key];
        end } );

local NPCS = L;
local OBJECTIVES = L;
local BUFFS = L;
local HOLIDAYS = L;
local QuestObjectiveComplete = Baldrick.QuestObjectiveComplete;

local function IsTargetAliveAndNotTapped()
    return ( not UnitIsDead( "playertarget" ) ) and
       ( UnitIsTappedByPlayer( "playertarget" ) or ( not UnitIsTapped( "playertarget" ) ) );
end--IsTargetAliveAndNotTapped()

local function IsTargetDead()
    return UnitIsDead( "playertarget" );
end--IsTargetDead()

local function IsPlayerOnTaxi()
    return UnitOnTaxi( "player" );
end--IsPlayerOnTaxi()

local function IsPlayerMountedOrBird()
   return IsMounted() or ( ( select( 2, UnitClass( "player" ) ) == "Druid" ) and ( select( GetShapeshiftFormInfo( 1, GetShapeshiftForm( true ) ) ):find( "Ability_Druid_FlightForm", 1, true ) ~= nil ) );
end--IsPlayerMountedOrBird()

local function IsPlayerPositionNear( x, y )
    local playerX, playerY = GetPlayerMapPosition( "player" );

    --return true if the player's position is in a 1x1 box around ( x, y )
    return ( playerX >= x - 0.005 ) and ( playerX <= x + 0.005 ) and
        ( playerY >= y - 0.005 ) and ( playerY <= y + 0.005 );
end--IsPlayerPositionNear( x, y )

local function TargetObjectiveCheckAlive( quest, questid, target )
    if( UnitIsDead( "playertarget" ) ) then
        return false;
    end--if
    
    local objectives = quest.objectives;
    
    for index, name in pairs( quest.targets ) do
        if( ( target:find( name, 1, true ) or name:find( target, 1, true ) ) and QuestObjectiveComplete( questid, objectives[index] ) ) then
            return false;
        end--if
    end--for
    return true;
end--TargetObjectiveCheckAlive( quest, questid, target )

local function TargetObjectiveCheckDead( quest, questid, target )
    if( not UnitIsDead( "playertarget" ) ) then
        return false;
    end--if
    
    local objectives = quest.objectives;
    
    for index, name in pairs( quest.targets ) do
        if( ( target:find( name, 1, true ) or name:find( target, 1, true ) ) and QuestObjectiveComplete( questid, objectives[index] ) ) then
            return false;
        end--if
    end--for
    return true;
end--TargetObjectiveCheckDead( quest, questid, target )

local function SubzoneObjectiveCheck( quest, questid, _, subzone )
    local objectives = quest.objectives;
    
    for index, name in pairs( quest.subZones ) do
        if( ( name == subzone ) and QuestObjectiveComplete( questid, objectives[index] ) ) then
            return false;
        end--if
    end--for
    return true;
end--TargetObjectiveCheckDead( quest, questid, target )

local function BuffNoHasCheck( quest )
    return not SpecialEventsAura:UnitHasBuff( "player", quest.buff );
end--BuffNoHasCheck( quest )

local function BuffNoHasCheck_DefaultItemName( quest )
    return not SpecialEventsAura:UnitHasBuff( "player", select( 1, GetItemInfo( quest.questItems.default.questItem ) ) );
end--BuffNoHasCheck_DefaultItemName( quest )

local function BuffHasCheck( quest )
    return SpecialEventsAura:UnitHasBuff( "player", quest.buff );
end--BuffHasCheck( quest )


local dailyQuests = {
    --[[
        These are the availiable fields; required are marked with a *, optional fields must be nil:
        questIDs: *Either a number with a questID or a table of numbers with questIDs.
        questItems: *A table of items required for the quest.  The one with an index of default is what will show on the button
                    The fields for items are:
                        questItem: the itemID of the item
                        questItemCount: the MINIMUM number required (the comparison will be >= )
        subZones: A string of subzones/zones this quest is valid for.  Zone names will be checked if in a null subzone.
                  Separate entries with |.
        target: String that has the name of a mob that must be the current target.  Separate entries with |.
        addionalEvents: Either a string with the name of an additional event that this quest should be checked when fired,
                        or a table of strings with names.  Events from a library that uses CallBackHandler can be registered for
                        buy using LibraryName:Event syntax.
        func: A function that takes two parameters.  The first parameter is the quest data iself, the second parameter will hold
              the questID being used.  If used this function must return a boolean, true if this quest is valid to show on the button.
        onEVENT: A function that will be ran whenever the event is encountered and the quest is registered for it.
        ]]--
    --Outland
    --Shattered Sun
    --Shattrath
    {
        --The Multiphase Survey
        questIDs = 11880,
        --Multiphase Spectrographic Goggles
        questItems = { default = { questItem = 35233, questItemCount = 1, }, },
        zones = ZONES["Nagrand"],
        subZones = { ZONES["Spirit Fields"], "", },
        additionalEvents = { "ClosetGnome_WearSet", "ClosetGnome_PartlyWearSet", };
        onClosetGnome_WearSet = "CloseGnomeFunc",
        onClosetGnome_PartlyWearSet = "CloseGnomeFunc",
        onQuest_Complete = "QuestFunc",
        onQuest_Fail = "QuestFunc",
        onQuest_Lost = "QuestFunc",
        onZONE_CHANGED = "ZoneFunc",
        onZONE_CHANGED_NEW_AREA = "ZoneFunc",
        -- Nonstandard stuffs
        lastSet = "";
        CloseGnomeFunc =
            function( quest, set )
                if( set ~= Baldrick.GetQuestName( quest.questIDs ) ) then
                    quest.lastSet = set;
                end--if
            end,
        QuestFunc =
            function( quest, questName, uid )
                if( ClosetGnome and ( uid == quest.questIDs ) and questName and ClosetGnome:HasSet( questName ) and ClosetGnome:IsSetFullyEquipped( questName ) ) then
                    ClosetGnome:WearSet( quest.lastSet );
                end--if
            end,
        ZoneFunc =
            function( quest )
                local questID = quest.questIDs;
                local questName = Baldrick.GetQuestName( questID );

                if( ClosetGnome and questName and Baldrick.HasQuestAndNotComplete( questID ) and ClosetGnome:HasSet( questName ) ) then
                    local subZone = GetSubZoneText();

                    if( ( subZone == ZONES["Spirit Fields"] ) or ( ( subZone == "" ) and ( quest.zones == GetRealZoneText() ) ) ) then
                        ClosetGnome:WearSet( questName );
                    else
                        ClosetGnome:WearSet( quest.lastSet );
                    end--if
                end--if
            end,
    },
    {
        --Intercepting the Mana Cells
        --Maintaining the Sunwell Portal
        questIDs = { 11513, 11514, },
        --Bash'ir Phasing Device
        questItems = { default = { questItem = 34248, questItemCount = 1, }, },
        zones = ZONES["Blade's Edge Mountains"],
        subZones = ZONES["Bash'ir Landing"],
        additionalEvents = { "LibLordFarlander-SpecialEvents-Aura-4.0:PlayerBuffGained", "LibLordFarlander-SpecialEvents-Aura-4.0:PlayerBuffLost" },
        func = BuffNoHasCheck_DefaultItemName,
    },
    --Quel'Danas
    {
        --Erratic Behavior
        --Further Conversions
        questIDs = { 11524, 11525, },
        --Attuned Crystal Cores
        questItems = { default = { questItem = 34368, questItemCount = 1, }, },
        targets = NPCS["Erratic Sentry"],
        func =
            function()
                return UnitIsDead( "playertarget" ) and
                    ( UnitIsTappedByPlayer( "playertarget" ) or ( not UnitIsTapped( "playertarget" ) ) );
            end,
    },
    {
        --The Sanctum Wards
        --Arm the Wards!
        questIDs = { 11496, 11523, },
        --Mana Remnants
        questItems = { default = { questItem = 34338, questItemCount = 4, }, },
        zones = ZONES["Isle of Quel'Danas"],
        subZones = ZONES["Sun's Reach Sanctum"],
    },
    {
        --The Battle for the Sun's Reach Armory
        --The Battle Must Go On
        questIDs = { 11538, 11537, },
        --Shattered Sun Banner
        questItems = { default = { questItem = 34414, questItemCount = 1, }, },
        targets = NPCS["Emissary of Hate"],
        func =
            function( _, questID )
                return UnitIsDead( "playertarget" ) and
                    ( ( not ( QuestObjectiveComplete( questID, OBJECTIVES["Emissary of Hate impaled"] ) ) ) and
                    ( not ( QuestObjectiveComplete( questID, OBJECTIVES["Emissary of Hate Impaled"] ) ) ) );
            end,
    },
    {
        --Distraction at the Dead Scar
        --The Air Strikes Must Continue
        questIDs = { 11532, 11533, },
        --Arcane Charges
        questItems = { default = { questItem = 34475, questItemCount = 1, }, },
        zones = ZONES["Isle of Quel'Danas"],
        subZones = ZONES["The Dead Scar"],
        additionalEvents = { "PLAYER_CONTROL_LOST", "PLAYER_CONTROL_GAINED", "TAXIMAP_CLOSED" },
        func = IsPlayerOnTaxi,
    },
    {
        --Disrupt the Greengill Coast
        questIDs = 11541,
        --Orb of Murloc Control
        questItems = { default = { questItem = 34483, questItemCount = 1, }, },
        targets = NPCS["Greengill Slave"],
        func = IsTargetAliveAndNotTapped,
    },
    {
        --Know Your Ley Lines
        questIDs = 11547,
        --Astromancer's Crystal
        questItems = { default = { questItem = 34533, questItemCount = 1, }, },
        zones = ZONES["Isle of Quel'Danas"],
        subZones = { ZONES["The Dawning Square"], ZONES["Dawnstar Village"], ZONES["Greengill Coast"], },
        func = SubzoneObjectiveCheck,
        --nonstandard
        objectives = {
            OBJECTIVES["Portal Reading Taken"],
            OBJECTIVES["Bloodcrystal Reading Taken"],
            OBJECTIVES["Shrine Reading Taken"],
        },
    },
    {
        --Intercept the Reinforcements
        --Keeping the Enemy at Bay
        questIDs = { 11542, 11543, },
        --Flaming Oil
        questItems = { default = { questItem = 34489, questItemCount = 1, }, },
        additionalEvents = { "PLAYER_CONTROL_LOST", "PLAYER_CONTROL_GAINED", "TAXIMAP_CLOSED", "ZONE_CHANGED" },
        func =
            function( _, questid )
                return UnitOnTaxi( "player" ) and
                    ( ( not QuestObjectiveComplete( questid, OBJECTIVES["Sin'loren sails burned"] ) ) or
                    ( not QuestObjectiveComplete( questid, OBJECTIVES["Bloodoath sails burned"] ) ) or
                    ( not QuestObjectiveComplete( questid, OBJECTIVES["Dawnchaser sails burned"] ) ) );
            end,
    },
    {
        --Ata'mal Armaments
        questIDs = 11544,
        --Ata'mal Armament
        questItems = { default = { questItem = 34500, questItemCount = 1, }, },
        zones = ZONES["Isle of Quel'Danas"],
        subZones = ZONES["Sun's Reach Armory"],
    },
    {
        --Discovering Your Roots
        --Rediscovering Your Roots
        questIDs = { 11520, 11521, },
        --Razorthorn Flayer Gland
        questItems = { default = { questItem = 34255, questItemCount = 1, }, },
        targets = NPCS["Razorthorn Ravager"],
        func = IsTargetAliveAndNotTapped,
    },
    --Throne of Kil'jaeden
    {
        --Blood for Blood
        questIDs = 11515,
        --Fel Siphon, Demonic Blood
        questItems = { default = { questItem = 34257, questItemCount = 1, }, { questItem = 34259, questItemCount = 1, }, },
        targets = NPCS["Felblood Initiate"],
        func = IsTargetAliveAndNotTapped,
    },
    {
        --Blast the Gateway
        questIDs = 11516,
        --Sizzling Embers
        questItems = { default = { questItem = 34253, questItemCount = 1, }, },
        zones = ZONES["Hellfire Peninsula"],
        subZones = ZONES["Throne of Kil'jaeden"],
    },

    --Skyguard
    --Skettis
    {
        --Fires Over Skettis
        questIDs = 11008,
        --Skyguard Blasting Charges
        questItems = { default = { questItem = 32406, questItemCount = 1, }, },
        zones = ZONES["Terokkar Forest"],
        subZones = { ZONES["Lower Veil Shil'ak"], ZONES["Skettis"], ZONES["Terokk's Rest"], ZONES["Upper Veil Shil'ak"], ZONES["Veil Ala'rak"], ZONES["Veil Harr'ik"], },
        additionalEvents = { "LibLordFarlander-SpecialEvents-Mount-4.0:Mounted", "LibLordFarlander-SpecialEvents-Mount-4.0:Dismounted", "UPDATE_SHAPESHIFT_FORM" },
        func = IsPlayerMountedOrBird,
    },
    --Blade's Edge Mountains
    {
        --Bombing Run
        --Bomb Them Again!
        questIDs = { 11010, 11102, 11023, },
        --Skyguard Bombs
        questItems = { default = { questItem = 32456, questItemCount = 1, }, },
        zones = ZONES["Blade's Edge Mountains"],
        subZones = { ZONES["Forge Camp: Terror"], ZONES["Forge Camp: Wrath"], },
        additionalEvents = { "LibLordFarlander-SpecialEvents-Mount-4.0:Mounted", "LibLordFarlander-SpecialEvents-Mount-4.0:Dismounted", "UPDATE_SHAPESHIFT_FORM" },
        func = IsPlayerMountedOrBird,
    },
    {
        --Wrangle Some Aether Rays!
        --Wrangle More Aether Rays!
        questIDs = { 11065, 11066, },
        --Wrangling Rope
        questItems = { default = { questItem = 32698, questItemCount = 1, }, },
        targets = NPCS["Aether Ray"],
        func = IsTargetAliveAndNotTapped,
    },

    --Ogri'la
    {
        --Banish The Demons
        --Banish More Demons
        questIDs = { 11026, 11051, },
        --Banishing Crystal, Apexis Shard
        questItems = { default = { questItem = 32696, questItemCount = 1, }, { questItem = 32569, questItemCount = 1, }, },
        zones = ZONES["Blade's Edge Mountains"],
        subZones = { ZONES["Forge Camp: Terror"], ZONES["Forge Camp: Wrath"], },
    },

    --Netherwing
    {
        --The Booterang: A Cure For The Common Worthless Peon
        questIDs = 11055,
        --Booterang
        questItems = { default = { questItem = 32680, questItemCount = 1, }, },
        targets = NPCS["Disobedient Dragonmaw Peon"],
        func = IsTargetAliveAndNotTapped,
    },
    {
        --A Slow Death
        questIDs = 11020,
        --Yarzill's Mutton
        --Fel Gland
        questItems = { default = { questItem = 32503, questItemCount = 1, }, { questItem = 32502, questItemCount = 1, }, },
        zones = ZONES["Netherstorm"],
        subZones = ZONES["Netherwing Ledge"],
    },

    --Northrend
    --Howling Fjord
    {
        --The Way to His Heart...
        questIDs = 11472,
        --Tasty Reef Fish
        questItems = { default = { questItem = 34127, questItemCount = 1, }, },
        targets = NPCS["Reef Bull"],
        func = IsTargetAliveAndNotTapped,
    },
    {
        --The Way to His Heart...
        questIDs = 11472,
        --Anuniaq's Net
        questItems = { default = { questItem = 40946, questItemCount = 1, }, },
        zones = ZONES["Howling Fjord"],
        subZones =  { "", ZONES["Garvan's Reef"], },
    },
    {
        --Break the Blockade
        questIDs = 11153,
        --Petrov's Cluster Bombs
        questItems = { default = { questItem = 33098, questItemCount = 1, }, },
        zones = ZONES["Howling Fjord"],
        subZones = ZONES["Shattered Straits"],
    },
    --Borean Tundra
    {
        --Drake Hunt
        questIDs = 11940,
        --Raelorasz's Spear
        questItems = { default = { questItem = 35506, questItemCount = 1, }, },
        targets = NPCS["Nexus Drake Hatchling"],
        func = IsTargetAliveAndNotTapped,
    },
    --Ahn'kahet: The Old Kingdom
    {
        --All Things in Good Time
        questIDs = 13190,
        --Ahn'kahar Watcher's Corpse
        questItems = { default = { questItem = 43494, questItemCount = 1, }, },
        zones = ZONES["Ahn'kahet: The Old Kingdom"],
    },
    --Grizzly Hills
    {
        --Smoke 'Em Out
        questIDs = { 12323, 12324, },
        --Smoke Bomb
        questItems = { default = { questItem = 37621, questItemCount = 1, }, },
        zones = ZONES["Grizzly Hills"],
        subZones = ZONES["Venture Bay"],
    },
    {
        --Seared Scourge
        questIDs = { 12029, 12038, },
        --Mack's Dark Grog
        questItems = { default = { questItem = 35908, questItemCount = 1, }, },
        zones = ZONES["Grizzly Hills"],
        subZones = ZONES["Drak'Tharon"],
    },
    --Zul'Drak
    {
        --Troll Patrol: Done to Death
        questIDs = 12568,
        --Incinerating Oil
        questItems = { default = { questItem = 38556, questItemCount = 1, }, },
        targets = NPCS["Defeated Argent Footman"],
        norange = true,
    },
    {
        --Troll Patrol: High Standards
        questIDs = 12502,
        --Argent Crusade Banner
        questItems = { default = { questItem = 38544, questItemCount = 1, }, },
        zones = ZONES["Zul'Drak"],
        subZones = ZONES["Drak'Sotra"],
    },
    {
        --Troll Patrol: Throwing Down
        questIDs = 12591,
        --High Impact Grenade
        questItems = { default = { questItem = 38574, questItemCount = 1, }, },
        zones = ZONES["Zul'Drak"],
        subZones = ZONES["Drak'Sotra"],
    },
    {
        --Troll Patrol: Can You Dig It?
        questIDs = 12588,
        --Steel Spade
        questItems = { default = { questItem = 38566, questItemCount = 1, }, },
        zones = ZONES["Zul'Drak"],
    },
    --Sholazar Basin
    {
        --A Hero's Headgear
        questIDs = 12758,
        --Venture Co. Explosives
        questItems = { default = { questItem = 39651, questItemCount = 1, }, },
        targets = NPCS["Stormwatcher"],
        func = IsTargetDead,
    },
    {
        --Rejek: First Blood
        questIDs = 12734,
        --Rejek's Blade
        questItems = { default = { questItem = 39577, questItemCount = 1, }, },
        targets = { NPCS["Hardnuckle Charger"], NPCS["Sapphire Hive Wasp"], NPCS["Mistwhisper"], },
        func = TargetObjectiveCheckDead,
        --nonstandard
        objectives = {
            OBJECTIVES["Blade blooded on Hardknuckle Charger"],
            OBJECTIVES["Blade blooded on Sapphire Hive Wasp"],
            OBJECTIVES["Blade Blooded on Mistwhisper members"],
        },
    },
    {
        --Power of the Great Ones
        questIDs = 12762,
        --Energized Polished Crystal
        questItems = { default = { questItem = 39748, questItemCount = 1, }, },
        additionalEvents = { "LibLordFarlander-SpecialEvents-Aura-4.0:PlayerBuffGained", "LibLordFarlander-SpecialEvents-Aura-4.0:PlayerBuffLost" },
        zones = ZONES["Sholazar Basin"],
        subZones = ZONES["Kartak's Hold"],
        func = BuffNoHasCheck,
        --nonstandard
        buff = BUFFS["Power of the Great Ones"];
    },
    {
        --The Heartblood's Strength
        questIDs = 12732,
        --Rejek's Vial
        questItems = { default = { questItem = 39574, questItemCount = 1, }, },
        zones = ZONES["Sholazar Basin"],
        subZones = ZONES["The Suntouched Pillar"],
    },
    {
        --Will of the Titans
        questIDs = 12705,
        --Tainted Crystal
        questItems = { default = { questItem = 39266, questItemCount = 1, }, },
        zones = ZONES["Sholazar Basin"],
        subZones = ZONES["Sparktouched Haven"],
    },
    {
        --Song of Fecundity
        questIDs = 12737,
        --Horn of Fecundity
        questItems = { default = { questItem = 39599, questItemCount = 1, }, },
        zones = ZONES["Sholazar Basin"],
        subZones = ZONES["The Stormwright's Shelf"],
    },
    {
        --Song of Reflection
        questIDs = 12736,
        --Didgeridoo of Contemplation
        questItems = { default = { questItem = 39598, questItemCount = 1, }, },
        zones = ZONES["Sholazar Basin"],
        subZones = { ZONES["The Glimmering Pillar"], ZONES["The Suntouched Pillar"], ZONES["The Mosslight Pillar"], ZONES["The Skyreach Pillar"], },
        func = SubzoneObjectiveCheck,
        --nonstandard
        objectives = {
            OBJECTIVES["Contemplation at Glimmering Pillar"],
            OBJECTIVES["Contemplation at Suntouched Pillar"],
            OBJECTIVES["Contemplation at Mosslight Pillar"],
            OBJECTIVES["Contemplation at Skyreach Pillar"],
        },
    },
    {
        --Song of Wind and Water
        questIDs = { 12726 },
        --Drums of the Tempest
        questItems = { default = { questItem = 39571, questItemCount = 1, }, },
        zones = ZONES["Sholazar Basin"],
        subZones = ZONES["The Stormwright's Shelf"],
    },
    {
        --Power of the Great Ones
        questIDs = 12762,
        --Dormant Polished Crystal
        questItems = { default = { questItem = 39747, questItemCount = 1, }, },
        zones = ZONES["Sholazar Basin"],
        subZones = ZONES["The Lifeblood Pillar"],
    },
    {
        --Kartak's Rampage
        questIDs = 12703,
        --Oracle Blood
        questItems = { default = { questItem = 39265, questItemCount = 1, }, },
        zones = ZONES["Sholazar Basin"],
        subZones = ZONES["Kartak's Hold"],
    },
    {
        --Appeasing the Great Rain Stone
        --Gods like Shiny Things
        questIDs = { 12704, 12572 },
        --Lafoo's Bug Bag
        questItems = { default = { questItem = 38622, questItemCount = 1, }, },
        zones = ZONES["Sholazar Basin"],
        subZones = { ZONES["Rainspeaker Canopy"], ZONES["Wildgrowth Mangal"], },
    },
    {
        --Appeasing the Great Rain Stone
        questIDs = { 12704 },
        --Jaloot's Favorite Crystal
        questItems = { default = { questItem = 38623, questItemCount = 1, }, },
        zones = ZONES["Sholazar Basin"],
        subZones = { ZONES["Rainspeaker Canopy"], ZONES["Wildgrowth Mangal"], },
    },
    {
        --Appeasing the Great Rain Stone
        questIDs = { 12704 },
        --Moodle's Stress Ball
        questItems = { default = { questItem = 38624, questItemCount = 1, }, },
        zones = ZONES["Sholazar Basin"],
        subZones = { ZONES["Rainspeaker Canopy"], ZONES["Wildgrowth Mangal"], },
    },
    {
        --A Cleansing Song
        questIDs = 12735,
        --Chime of Cleansing
        questItems = { default = { questItem = 39572, questItemCount = 1, }, },
        zones = ZONES["Sholazar Basin"],
        subZones = { ZONES["River's Heart"], ZONES["Wintergrasp River"], ZONES["Bittertide Lake"], },
        func = SubzoneObjectiveCheck,
        --nonstandard
        objectives = {
            OBJECTIVES["Spirit of Ha-Khalan slain"],
            OBJECTIVES["Spirit of Koosu slain"],
            OBJECTIVES["Spirit of Atha slain"],
        },
    },
    {
        --Secret Strength of the Frenzyheart
        questIDs = 12760,
        --Wolvar Berries
        --Thunderbrew's Hard Ale
        questItems = { default = { questItem = 39739, questItemCount = 1, }, { questItem = 39738, questItemCount = 1, }, },
    },
    {
        --Secret Strength of the Frenzyheart
        questIDs = 12760,
        --Secret Strength of the Frenzyheart
        questItems = { default = { questItem = 39737, questItemCount = 1, }, },
        zones = ZONES["Sholazar Basin"],
        additionalEvents = { "LibLordFarlander-SpecialEvents-Aura-4.0:PlayerBuffGained", "LibLordFarlander-SpecialEvents-Aura-4.0:PlayerBuffLost" },
        func = BuffNoHasCheck_DefaultItemName,
    },
    {
        --Chicken Party!
        --Flown the Coop!
        questIDs = { 12702, 12532 },
        --Chicken Net
        questItems = { default = { questItem = 38689, questItemCount = 1, }, },
        zones = ZONES["Sholazar Basin"],
    },
    --The Storm Peaks
    {
        --Maintaining Discipline
        --Discipline
        questIDs = { 13422, 12906 },
        --Disciplining Rod
        questItems = { default = { questItem = 42837, questItemCount = 1, }, },
        targets = NPCS["Exhausted Vrykul"],
        func = IsTargetAliveAndNotTapped,
    },
    {
        --Blowing Hodir's Horn
        questIDs = 12977,
        --Hodir's Horn
        questItems = { default = { questItem = 42164, questItemCount = 1, }, },
        targets = { NPCS["Niffelem Forefather"], NPCS["Restless Frostborn"], },
        func = TargetObjectiveCheckDead,
        --nonstandard
        objectives = {
            OBJECTIVES["Niffelem Forefather Freed"],
            OBJECTIVES["Restless Frostborn Freed"],
        },
    },
    {
        --Feeding Arngrim
        questIDs = 13046,
        --Arngrim's Tooth
        questItems = { default = { questItem = 42774, questItemCount = 1, }, },
        targets = NPCS["Roaming Jormungar"],
        func = IsTargetAliveAndNotTapped,
    },
    {
        --Thrusting Hodir's Spear
        questIDs = 13003,
        --Hodir's Spear
        questItems = { default = { questItem = 42769, questItemCount = 1, }, },
        targets = NPCS["Wild Wyrm"],
        func = IsTargetAliveAndNotTapped,
    },
    {
        --Spy Hunter
        questIDs = 12994,
        --Ethereal Worg's Fang
        questItems = { default = { questItem = 42479, questItemCount = 1, }, },
        zones = ZONES["The Storm Peaks"],
        subZones = ZONES["Valley of Ancient Winters"],
    },
    {
        --Hot and Cold
        questIDs = 12981,
        --Essence of Ice
        questItems = { default = { questItem = 42246, questItemCount = 1, }, },
        zones = ZONES["The Storm Peaks"],
        subZones = ZONES["Fjorn's Anvil"],
    },
    {
        --Overstock
        questIDs = 12833,
        --Improved Land Mines
        questItems = { default = { questItem = 40676, questItemCount = 1, }, },
        zones = ZONES["The Storm Peaks"],
        subZones = ZONES["Garm's Bane"],
    },
    {
        --Back to the Pit
        --Into the Pit
        questIDs = { 13424, 12997 },
        --Reins of the Icemaw Matriarch
        questItems = { default = { questItem = 42499, questItemCount = 1, }, },
        zones = ZONES["The Storm Peaks"],
        subZones = ZONES["The Pit of the Fang"],
        func = IsTargetAliveAndNotTapped,
    },
    {
        --The Aberrations Must Die
        --Aberrations
        questIDs = { 13425, 12925 },
        --Vial of Frost Oil
        questItems = { default = { questItem = 41612, questItemCount = 1, }, },
        zones = ZONES["The Storm Peaks"],
        subZones = ZONES["The Blighted Pool"],
    },
    --Icecrown
    {
        --Drag and Drop
        questIDs = { 13323, 13353, 13318, 13352, },
        --Orb of Illusion
        questItems = { default = { questItem = 44246, questItemCount = 1, }, },
        targets = NPCS["Dark Subjugator"],
        func = IsTargetAliveAndNotTapped,
    },
    {
        --Life or Death
        questIDs = 12296,
        --Renewing Bandage
        questItems = { default = { questItem = 37576, questItemCount = 1, }, },
        targets = NPCS["Wounded Westfall Infantry"],
        func = IsTargetAliveAndNotTapped,
    },
    {
        --Overwhelmed!
        questIDs = 12288,
        --Renewing Tourniquet
        questItems = { default = { questItem = 37568, questItemCount = 1, }, },
        targets = NPCS["Wounded Skirmisher"],
        func = IsTargetAliveAndNotTapped,
    },
    {
        --Leave Our Mark!
        questIDs = 12995,
        --Ebon Blade Banner
        questItems = { default = { questItem = 42480, questItemCount = 1, }, },
        targets = { NPCS["Jotunheim Warrior"], NPCS["Njorndar Spear-Sister"], NPCS["Valhalas Vargul"], NPCS["Mjordin Water Magus"], },
        func = IsTargetDead,
    },
    {
        --Not a Bug
        questIDs = { 13344, 13365, 13358, 13342, },
        --Rod of Siphoning
        questItems = { default = { questItem = 44009, questItemCount = 1, }, },
        targets = NPCS["Enslaved Minion"],
        func = IsTargetDead,
    },
    {
        --From Their Corpses, Rise!
        questIDs = 12813,
        --Darkmender's Tincture
        questItems = { default = { questItem = 40587, questItemCount = 1, }, },
        targets = NPCS["Onslaught"],
        zone = ZONES["Icecrown"],
        subZones = ZONES["Onslaught Harbor"],
        func = IsTargetDead,
    },
    {
        --No Fly Zone
        questIDs = 12815,
        --Bone Gryphon
        questItems = { default = { questItem = 40600, questItemCount = 1, }, },
        zone = ZONES["Icecrown"],
        subZones = ZONES["Onslaught Harbor"],
        additionalEvents = { "LibLordFarlander-SpecialEvents-Aura-4.0:PlayerBuffGained", "LibLordFarlander-SpecialEvents-Aura-4.0:PlayerBuffLost" },
        func = BuffNoHasCheck_DefaultItemName,
    },
    {
        --The Solution Solution
        questIDs = 13292,
        --Smuggled Solution, Abandoned Helm, Abandoned Armor, Pile of Bones
        questItems = { default = { questItem = 44048, questItemCount = 1, }, { questItem = 43610, questItemCount = 1, }, { questItem = 43616, questItemCount = 1, }, { questItem = 43609, questItemCount = 1, }, },
        zone = ZONES["Icecrown"],
        subZones = ZONES["Broken Front"],
    },
    {
        --Volatility
        questIDs = 13261,
        --Copperclaw's Volatile Oil, Abandoned Helm, Abandoned Armor, Pile of Bones
        questItems = { default = { questItem = 43608, questItemCount = 1, }, { questItem = 43610, questItemCount = 1, }, { questItem = 43616, questItemCount = 1, }, { questItem = 43609, questItemCount = 1, }, },
        zone = ZONES["Icecrown"],
        subZones = ZONES["Broken Front"],
    },
    {
        --Neutralizing the Plague
        questIDs = { 13281, 13297, },
        --Pustulant Spinal Fluid
        questItems = { default = { questItem = 44010, questItemCount = 1, }, },
        zone = ZONES["Icecrown"],
        subZones = ZONES["Mord'rethar"],
    },
    {
        --Retest Now
        questIDs = { 13322, 13357, 13321, 13356, },
        --Writhing Mass
        questItems = { default = { questItem = 44304, questItemCount = 1, }, },
        zone = ZONES["Icecrown"],
        subZones = ZONES["Aldur'Thar: The Desolation Gate"],
    },
    {
        --That's Abominable!
        questIDs = { 13264, 13276, 13288, 13289, },
        --Abomination Reanimation Kit, Chilled Abomination Guts
        questItems = { default = { questItem = 43968, questItemCount = 1, }, { questItem = 43966, questItemCount = 1, }, },
    },
    {
        --Neutralizing the Plague
        questIDs = { 13281, 13297, },
        --Flesh Giant Spine
        questItems = { default = { questItem = 44009, questItemCount = 1, }, },
    },
    {
        --Retest Now
        questIDs = { 13322, 13357, 13321, 13356, },
        --Diluted Cult Tonic
        questItems = { default = { questItem = 44307, questItemCount = 1, }, },
        additionalEvents = { "LibLordFarlander-SpecialEvents-Aura-4.0:PlayerBuffGained", "LibLordFarlander-SpecialEvents-Aura-4.0:PlayerBuffLost" },
        zone = ZONES["Icecrown"],
        subZones = ZONES["Aldur'Thar: The Desolation Gate"],
        func = BuffNoHasCheck,
        --nonstandard
        buff = BUFFS["Dark Discernment"],
    },
    {
        --Retest Now
        questIDs = { 13322, 13357, 13321, 13356, },
        --Tainted Essence
        questItems = { default = { questItem = 44301, questItemCount = 10, }, },
    },

    --Argent Tournament
    -- { --Retired
        -- --A Chip Off the Ulduar Block
        -- questIDs = 13681,
        -- --Goblin Chisel
        -- questItems = { default = { questItem = 45281, questItemCount = 1, }, },
        -- zones = ZONES["The Storm Peaks"],
    -- },
    -- { --Retired
        -- --Jack Me Some Lumber
        -- questIDs = 13627,
        -- --Lumberjack's Axe
        -- questItems = { default = { questItem = 45046, questItemCount = 1, }, },
        -- targets = NPCS["Crystalsong Oak"],
    -- },
    {
        --The Edge Of Winter
        questIDs = { 13603, 13670, 13675, 13743, 13748, 13754, 13754, 13759, 13616, 13764, 13770, 13775, 13780, 13785 },
        --Everburning Ember
        questItems = { default = { questItem = 45005, questItemCount = 1, }, },
        targets = NPCS["Maiden of Winter's Breath"],
    },
    {
        --A Worthy Weapon
        questIDs = { 13600, 13669, 13674, 13742, 13747, 13753, 13758, 13763, 13769, 13774, 13779, 13784 },
        --Winter Hyacinth
        questItems = { default = { questItem = 45000, questItemCount = 4, }, },
        zones = ZONES["Dragonblight"],
        subZones = ZONES["Drak'Mar Lake"],
    },
    {
        --A Blade Fit For A Champion
        questIDs = { 13603, 13666, 13673, 13741, 13746, 13752, 13757, 13762, 13768, 13773, 13778, 13783, },
        --Warts-B-Gone Lip Balm
        questItems = { default = { questItem = 44986, questItemCount = 1, }, },
        targets = NPCS["Lake Frog"],
        func = BuffNoHasCheck,
        --nonstandard
        buff = BUFFS["Warts-B-Gone Lip Balm"],
    },
    {
        --A Blade Fit For A Champion
        questIDs = { 13603, 13666, 13673, 13741, 13746, 13752, 13757, 13762, 13768, 13773, 13778, 13783, },
        --/kiss
        questItems = { default = { questAction = { macro = "/kiss", icon = [[Interface\AddOns\Baldrick\Img\Kiss]], description = L["Kiss"], }, }, },
        targets = NPCS["Lake Frog"],
        func = BuffHasCheck,
        --nonstandard
        buff = BUFFS["Warts-B-Gone Lip Balm"],
    },
    --3.2
    {
        --Breakfast Of Champions
        questIDs = { 14076, 14092, },
        --Earthshaker Drum
        questItems = { default = { questItem = 46893, questItemCount = 1, }, },
        zones = ZONES["The Storm Peaks"],
        subZones = ZONES["The Foot Steppes"],
    },
    {
        --Get Kraken!
        questIDs = { 14108, },
        --Flaming Spears
        questItems = { default = { questItem = 47006, questItemCount = 1, }, },
        targets = { NPCS["Kvaldir Deepcaller"], NPCS["North Sea Kraken"], },
        norange = true,
        func = TargetObjectiveCheckAlive,
        --nonstandard
        objectives = {
            OBJECTIVES["Kvaldir Deepcaller slain"],
            OBJECTIVES["Hurl Spears at North Sea Kraken"],
        },
    },
    {
        --The Light's Mercy
        questIDs = { 14077, 14144, },
        --Confessor's Prayer Book
        questItems = { default = { questItem = 46870, questItemCount = 1, }, },
        targets = NPCS["Slain Tualiq Villager"],
        norange = true,
    },
    {
        --Drottinn Hrothgar
        questIDs = { 14101, },
        --Kvaldir War Horn
        questItems = { default = { questItem = 47006, questItemCount = 1, }, },
        zones = ZONES["Hrothgar's Landing"],
    },
    {
        --Mistcaller Yngvar
        questIDs = { 14102, },
        --Mistcaller's Charm
        questItems = { default = { questItem = 47009, questItemCount = 1, }, },
        zones = ZONES["Hrothgar's Landing"],
    },
    {
        --Ornolf The Scarred
        questIDs = { 14104, },
        --Captured Kvaldir Banner
        questItems = { default = { questItem = 47029, questItemCount = 1, }, },
        zones = ZONES["Hrothgar's Landing"],
    },
    {
        --Gormok Wants His Snobolds
        questIDs = { 14090, 14141, },
        --Weighted Net
        questItems = { default = { questItem = 46885, questItemCount = 1, }, },
        targets = NPCS["Snowblind Follower"],
        func = IsTargetAliveAndNotTapped,
        norange = true,
    },
    {
        --The Fate Of The Fallen
        questIDs = { 14107, },
        --Light-Blessed Relic, Discarded Soul Crystal
        questItems = { default = { questItem = 47033, questItemCount = 1, }, { questItem = 47035, questItemCount = 1, }, },
        targets = NPCS["Fallen Hero's Spirit"],
        func = IsTargetAliveAndNotTapped,
    },
    {
        --What Do You Feed a Yeti, Anyway?
        questIDs = { 14112, 14145, },
        --Fresh Chum
        questItems = { default = { questItem = 47036, questItemCount = 1, }, },
        func = IsSwimming,
    },

    --Holiday
    --Noblegarden
    {
        --The Great Egg Hunt
        questIDs = { 13479, 13480, },
        --Brightly Colored Egg
        questItems = { default = { questItem = 45072, questItemCount = 1, } },
        holiday = HOLIDAYS["Noblegarden"],
    },
    --Midsummer Fire Festival
    {
        --Torch Catching
        --More Torch Catching
        questIDs = { 11657, 11923, 11924, 11925, },
        --Unlit Torches
        questItems = { default = { questItem = 34833, questItemCount = 1, }, },
        holiday = HOLIDAYS["Midsummer Fire Festival"],
    },
    {
        --Torch Tossing
        --More Torch Tossing
        questIDs = { 11731, 11922, 11921, 11926, },
        --Practice Torches
        questItems = { default = { questItem = 34862, questItemCount = 1, }, },
        holiday = HOLIDAYS["Midsummer Fire Festival"],
    },
    --Brewfest
    {
        --Now This is Ram Racing... Almost.
        --There and Back Again
        --Bark for Drohn's Distillery!
        --Bark for T'chali's Voodoo Brewery!
        --Bark for the Barleybrews!
        --Bark for the Thunderbrews!
        questIDs = { 11318, 11409, 11122, 11412, 11407, 11408, 11293, 11294, },
        --Ram Racing Reins
        questItems = { default = { questItem = 33306, questItemCount = 1, }, },
        holiday = HOLIDAYS["Brewfest"],
        additionalEvents = { "LibLordFarlander-SpecialEvents-Aura-4.0:PlayerBuffGained", "LibLordFarlander-SpecialEvents-Aura-4.0:PlayerBuffLost" },
        func = BuffHasCheck,
        --nonstandard
        buff = BUFFS["Rental Racing Ram"],
    },
    --Hallow's End
    {
        --Fire Brigade Practice
        --Fire Training
        --Stop the Fires!
        --"Let the Fires Come!"
        questIDs = { 11360, 11439, 11440, 11361, 11449, 11450, 11131, 11219, 12135, 12139, },
        --Water Bucket
        questItems = { default = { questItem = 32971, questItemCount = 1, }, },
        holiday = HOLIDAYS["Hallow's End"],
    },

    --Cooking
    --Northrend
    {
        --Cheese for Glowergold
        questIDs = { 13103, 13115, },
        --Empty Cheese Serving Platter, Aged Dalaran Cheese, Half Full Wine Glass
        questItems = { default = { questItem = 43139, questItemCount = 1, }, { questItem = 43137, questItemCount = 1, }, { questItem = 43138, questItemCount = 6, }, },
    },
    {
        --Infused Mushroom Meatloaf
        questIDs = { 13100, 13112, },
        --Meatloaf Pan, Infused Mushroom, Chilled Meat
        questItems = { default = { questItem = 43101, questItemCount = 1, }, { questItem = 43100, questItemCount = 4, }, { questItem = 43013, questItemCount = 2, }, },
    },
    {
        --Mustard Dogs!
        questIDs = { 13107, 13116, },
        --Empty Picnic Basket, Rhino Dog, Mustard Herb
        questItems = { default = { questItem = 43142, questItemCount = 1, }, { questItem = 34752, questItemCount = 4, }, { questItem = 43143, questItemCount = 4, }, },
    },
    {
        --Sewer Stew
        questIDs = { 13102, 13114, },
        --Stew Cookpot, Chilled Meat, Crystal Song Carrot
        questItems = { default = { questItem = 43147, questItemCount = 1, }, { questItem = 43013, questItemCount = 4, }, { questItem = 43148, questItemCount = 4, }, },
    },
    --Outland
    {
        --Revenge is Tasty
        questIDs = 11377,
        --Cooking Pot: Kaliri Stew, Giant Kaliri Wing, Warp Burger
        questItems = { default = { questItem = 33837, questItemCount = 1, }, { questItem = 33838, questItemCount = 1, }, { questItem = 27659, questItemCount = 3, }, },
    },
    {
        --Soup for the Soul
        questIDs = 11381,
        --Cooking Pot: Spiritual Soup, Roasted Clefthoof
        questItems = { default = { questItem = 33851, questItemCount = 1, }, { questItem = 27658, questItemCount = 4, }, },
        zones = ZONES["Nagrand"],
        subZones = ZONES["Ancestral Grounds"],
    },
    {
        --Super Hot Stew
        questIDs = 11379,
        --Cooking Pot: Demon Broiled Surprise, Mok'Nathal Shortribs, Crunchy Serpent
        questItems = { default = { questItem = 33852, questItemCount = 1, }, { questItem = 31672, questItemCount = 2, }, { questItem = 31673, questItemCount = 1, }, },
        targets = NPCS["Abyssal Flamebringer"],
        func = IsTargetDead,
    },
    
    --Fishing
    --Northrend
    {
        --The Ghostfish
        questIDs = 13830,
        --Phantom Ghostfish
        questItems = { default = { questItem = 45902, questItemCount = 1, }, },
    },
    {
        --Monsterbelly Appetite
        questIDs = 13836,
        --Bloated Monsterbelly
        questItems = { default = { questItem = 45328, questItemCount = 1, }, },
    },
    --Outland
    {
        --Shrimpin' Ain't Easy
        questIDs = 11668,
        --Bloated Barbed Gill Trout
        questItems = { default = { questItem = 35313, questItemCount = 1, }, },
    },
};

Baldrick:RegisterQuestModule( L["Daily Quests"], dailyQuests, nil, 2 );
